#!/usr/bin/env bash
#===============================================================================
#
#          FILE:  odm-attributes.sh
#
#         USAGE:  ./odm-attributes.sh '<json_attribute>'
#         DEBUG:  TRACE=1 ./odm-attributes.sh '<json_attribute>'
#
#   DESCRIPTION: Executes functions after received attribute updates from the ODM. It is called by the odm-agent.
#
#       OPTIONS:  ---
#  REQUIREMENTS:  ---
#          BUGS:  ---
#        AUTHOR:  owzgra, owzppi
#       COMPANY:  Owasys SL
#       VERSION:  0.1.0
#       CREATED:  2024-02-23
#===============================================================================
##### LOGGER

LOGGER="/usr/bin/logger"
LOG_LABEL="odm-attributes"
$LOGGER -t "$LOG_LABEL" "Starting odm-attributes.sh"
set -o errexit
set -o nounset
set -o pipefail
if [[ "${TRACE-0}" == "1" ]]; then
    set -o xtrace
fi

if [[ "${1-}" =~ ^-*h(elp)?$ ]]; then
    echo $"Usage: ./odm-attributes.sh <JSON>
Executes functions after received attribute updates from the ODM. It is called by the odm-agent.
"
    exit
fi

cd "$(dirname "$0")"

################################# configuration vars #############################

##### PLATFORM

PLATFORM='unknown'
MACHINE_TYPE=$(uname -m)
if [ "${MACHINE_TYPE}" = "armv5tejl" ]; then
  PLATFORM='owa3x'
elif [ "${MACHINE_TYPE}" = "armv7l" ]; then
  PLATFORM='owa4x'
elif [ "${MACHINE_TYPE}" = "x86_64" ]; then
  PLATFORM='x86_64'
fi

printf "Running on %s platform\n" $PLATFORM >&2

##### PLATFORM-SPECIFIC VARIABLES

JQ="/usr/bin/jq"

################################# attribute functions #############################

namespacePrefix(){
    $LOGGER -t "$LOG_LABEL" "namespacePrefix attribute received"
    json_file="/device/odm/odm.json"
    temp_json_file="/tmp/odm.json"
    if [ -f "$json_file" ]; then
        nsp=$(jq -r '.namespacePrefix' $json_file)
        if [ "$nsp" != "$1" ]; then
            jq --arg nsp  "$1"  '.namespacePrefix=$nsp' "$json_file" > "$temp_json_file" && mv "$temp_json_file" "$json_file" && sync "$json_file"
            $LOGGER -t "$LOG_LABEL" "namespacePrefix set successfully in $json_file"
        else
            $LOGGER -t "$LOG_LABEL" "namespacePrefix has not changed in $json_file"
        fi
  else
    $LOGGER -t "$LOG_LABEL" "Failed setting namespacePrefix, file $json_file does not exist"
  fi
}

vpn_settings( ){
    $LOGGER -t "$LOG_LABEL" "vpn_settings attribute received"
    address=$( echo $shared_values | $JQ -r '.address' )
    if( [ $address == "null" ] ); then
        $LOGGER -t "$LOG_LABEL" "Wireguard stopped"
        wg-quick down wg0
        rm /etc/wireguard/wg0.conf
        return 0
    fi
    ips=$( echo $shared_values | $JQ -r '.allowedIPs' )
    endpoint=$( echo $shared_values | $JQ -r '.endpoint' )
    publicKey=$( echo $shared_values | $JQ -r '.serverPublicKey' )
    sharedKey=$( echo $shared_values | $JQ -r '.preSharedKey' )
    privateKey=$( echo $shared_values | $JQ -r '.privateKey' )
    keepalive=$( echo $shared_values | $JQ -r '.persistentKeepalive' )
    wg="[Interface]
PrivateKey = ${privateKey}
Address = ${address}
MTU = 1000
PostUp = redis-cli rpush tb-telemetry '{\"connected_vpn\": 1}'
PostDown = redis-cli rpush tb-telemetry '{\"connected_vpn\": 0}'
[Peer]
PublicKey = ${publicKey}
AllowedIPs = ${ips}
Endpoint = ${endpoint}
PresharedKey = ${sharedKey}
PersistentKeepalive = ${keepalive}"
    echo "$wg" > /etc/wireguard/wg0.conf
    wg-quick up wg0
    $LOGGER -t "$LOG_LABEL" "Wireguard configuration set successfully"    
}
################################# aux functions #############################

parse()
{
    ret=0
    shared_key=$(echo "$1" | $JQ  -r 'keys[]')
    shared_values=$(echo "$1" | $JQ  -r 'values[]')
    $LOGGER -t "$LOG_LABEL" "Parsing shared key: $shared_key"
    case $shared_key in
        "namespacePrefix")
            namespacePrefix "$shared_values"
            ;;
        "vpn_settings")
            vpn_settings "$shared_values"
            ;;
        *)
            $LOGGER -t "$LOG_LABEL" "ERROR: unknown shared key \"$shared_key\""
            printf "Nothing to do here...\n" >&2
            ret=1
            ;;

    esac
    return $ret
}

################################# main function #############################

main() {    
    JSON_ATTRIBUTE=$1
    parse "$JSON_ATTRIBUTE"
    printf "Attribute processed\n" >&2
}

main "$@"